package net.udby.test.sdf;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.TimeZone;

public class DateFormatUtilThreadLocal {
	/**
	 * Helper class for "thread-safe" SimpleDateFormat'ers, holding them in a ThreadLocal ensures they are not called from different threads
	 * at the same time without resorting to synchronization
	 */
	static class FormattersThreadCache extends ThreadLocal<Map<SdfCacheKey, SimpleDateFormat>> {
		@Override
		protected Map<SdfCacheKey, SimpleDateFormat> initialValue() {
			return new HashMap<SdfCacheKey, SimpleDateFormat>();
		}

		public SimpleDateFormat get(String formatString, Locale locale, TimeZone timeZone) {
			Map<SdfCacheKey, SimpleDateFormat> map = get();
			SdfCacheKey key = SdfCacheKey.c(formatString, locale, timeZone);
			SimpleDateFormat sdf = map.get(key);
			if (sdf == null) {
				sdf = new SimpleDateFormat(formatString, locale);
				sdf.setTimeZone(timeZone);
				map.put(key, sdf);
			}
			return sdf;
		}
	};

	/** ThreadLocal with SimpleDateFormat'ers */
	private static final FormattersThreadCache fmtCache = new FormattersThreadCache();

	private DateFormatUtilThreadLocal() {
	}

	public static String format(Date date, String formatString, Locale locale, TimeZone timeZone) {
		return fmtCache.get(formatString, locale, timeZone).format(date);
	}

	public static String format(Date date, String formatString, Locale locale) {
		return format(date, formatString, locale, TimeZone.getDefault());
	}

	public static String format(Date date, String formatString) {
		return format(date, formatString, Locale.getDefault());
	}

	public static Date parse(String date, String formatString, Locale locale, TimeZone timeZone) throws ParseException {
		return fmtCache.get(formatString, locale, timeZone).parse(date);
	}

	public static Date parse(String date, String formatString, Locale locale) throws ParseException {
		return parse(date, formatString, locale, TimeZone.getDefault());
	}

	public static Date parse(String date, String formatString) throws ParseException {
		return parse(date, formatString, Locale.getDefault());
	}
}

final class SdfCacheKey {
	final String formatString;
	final Locale locale;
	final TimeZone timeZone;

	private SdfCacheKey(String formatString, Locale locale, TimeZone timeZone) {
		super();
		this.formatString = formatString;
		this.locale = locale;
		this.timeZone = timeZone;
	}

	static SdfCacheKey c(String formatString, Locale locale, TimeZone timeZone) {
		return new SdfCacheKey(formatString, locale, timeZone);
	}

	private volatile int cachedHash = -1;

	@Override
	public int hashCode() {
		if (cachedHash == -1) {
			final int prime = 31;
			int result = 1;
			result = prime * result + ((formatString == null) ? 0 : formatString.hashCode());
			result = prime * result + ((locale == null) ? 0 : locale.hashCode());
			result = prime * result + ((timeZone == null) ? 0 : timeZone.hashCode());
			cachedHash = result;
		}
		return cachedHash;
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) {
			return true;
		}
		if (!(o instanceof SdfCacheKey)) {
			return false;
		}
		final SdfCacheKey other = (SdfCacheKey) o;
		boolean eq = (this.formatString == other.formatString || (this.formatString != null && this.formatString.equals(other.formatString)));
		eq = eq && (this.locale == other.locale || (this.locale != null && this.locale.equals(other.locale)));
		eq = eq && (this.timeZone == other.timeZone || (this.timeZone != null && this.timeZone.equals(other.timeZone)));
		return eq;
	}

	@Override
	public String toString() {
		return super.toString() + "[formatString=" + formatString + ",locale=" + locale + ",timeZone=" + timeZone + "]";
	}
}