package net.udby.test.sdf.simple;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.udby.test.sdf.ReportGenerator;
import net.udby.test.sdf.log.FirstFailure;
import net.udby.test.sdf.wrapper.stat.StatInfo;

public class ReportSimpleDriver {

	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception {

		List<Class<? extends SimpleApi>> testClasses = new ArrayList<Class<? extends SimpleApi>>();
		testClasses.add(SdfSimpleSync.Factory.class);
		testClasses.add(SdfSimpleCreate.Factory.class);
		testClasses.add(SdfSimpleClone.Factory.class);
		testClasses.add(SdfSimpleThreadLocal.Factory.class);

		// warmup, run every test with one thread for a second
		System.out.println("warming up...");
		for (Class<? extends SimpleApi> testClass : testClasses) {
			runTest(testClass, 1, 1000l, 0);
		}

		System.out.println("preparing report...");
		final String fileName = "simple-report-" + System.currentTimeMillis() + ".csv";
		PrintWriter pw = new PrintWriter(fileName);
		try {
			ReportGenerator report = new ReportGenerator(pw);
			report.headline("Report, Simple Synched SimpleDateFormat");
			report.emptyline();
			runTests(report, testClasses, 4, 10000l, 0);
			runTests(report, testClasses, 4, 10000l, 1l);
		} finally {
			pw.close();
		}
	}

	private static void runTests(ReportGenerator report, List<Class<? extends SimpleApi>> testClasses, int cnt, long runtime, long relaxtime)
			throws Exception {
		for (Class<? extends SimpleApi> testClass : testClasses) {
			report.statHeadline(testClass, cnt, runtime, relaxtime);
			System.out.printf("%s, threads=%d, runtime(ms)=%d, relaxtime(ms)=%d%n", testClass.getName(), cnt, runtime, relaxtime);
			Map<String, StatInfo> map = runTest(testClass, cnt, runtime, relaxtime);
			if (map == null) {
				report.failline(FirstFailure.getInstance().toString());
				FirstFailure.clear();
			} else {
				for (StatInfo si : map.values()) {
					report.statline(si);
				}
			}
		}
	}

	private static <T extends SimpleApi> Map<String, StatInfo> runTest(Class<T> clazz, int cnt, long runtime, long relaxtime)
			throws InterruptedException {
		final Object sync = new Object();
		List<TestRunner<T>> runners = new ArrayList<TestRunner<T>>(cnt);
		List<Thread> threads = new ArrayList<Thread>(cnt);
		for (int i = 0; i < cnt; i++) {
			TestRunner<T> runner = new TestRunner<T>(clazz, runtime, relaxtime, sync);
			runners.add(runner);
			Thread thread = new Thread(runner, "testing-" + clazz.getSimpleName() + "-" + (i + 1));
			threads.add(thread);
		}
		synchronized (sync) {
			for (Thread thread : threads) {
				thread.start();
			}
			sync.wait(200l);
			sync.notifyAll();
		}
		for (Thread thread : threads) {
			thread.join();
		}

		if (FirstFailure.failed()) {
			return null;
		}

		Map<String, StatInfo> methodStats = new HashMap<String, StatInfo>();
		for (TestRunner<T> runner : runners) {
			for (Map.Entry<String, StatInfo> entry : runner.getHandler().methodStats.entrySet()) {
				StatInfo si = methodStats.get(entry.getKey());
				if (si == null) {
					methodStats.put(entry.getKey(), entry.getValue());
				} else {
					methodStats.put(entry.getKey(), StatInfo.consolidate(entry.getValue(), si));
				}
			}
		}

		return methodStats;
	}
}
